package org.greenstone.gatherer.cdm;
/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.cdm.Classifier;
import org.greenstone.gatherer.file.FileNode;
/** The interface for any custom classifier.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
public interface CustomClassifier
    extends Comparable {
    /** Constructor.
     * public CustomClassifier(Gatherer gatherer);
	  */
    /** Create a blank copy of this pseudo-classifier.
     * @return A newly allocated <strong>CustomClassifier</strong>.
     */
    public CustomClassifier copy();
    /** Destructor. */
    public void destroy();
    /** Show the controls for configuring this pseudo-classifier.
     * @param show <i>true</i> if the component should immediately show itself, <i>false</i> if you just wish to initialize components.
     */
    public void hide();
    public boolean display(boolean show);
    /** Method to return this pseudo-classifier represented as a String.
     * @return A <strong>String</strong>.
     */
    public String getCommand();
    /** Retrieve the custom command, a command line that overrides and replaces some other 'actual' classifier.
     * @param index The number of the classifer this one is replacing.
     */
    public String getCustomCommand(int index);
    /** Get the name of this custom classifier.
     * @return A <strong>String</strong> representing the name.
     */
    public String getName();
    public void process(FileNode record);
    /** Recreate a CustomAZList given several parameters including the real classifier created during custom design.
     * @param classifier The real <strong>Classifier</strong>.
     * @param separations A <strong>String</strong> representing the choosen separations.
     */
    public void recreate(Classifier classifier, String separations);
    /** Sets the value of Gatherer, for those classes loaded dynamically.
     * @param gatherer A reference to the <strong>Gatherer</strong>.
     */
    public void setGatherer(Gatherer gatherer);
    /** Sets the Classifier manager in charge of this classifier.
     * @param manager This classifiers <strong>ClassifierManager</strong>.
     */
    public void setManager(ClassifierManager manager);
    
    /** Determines if the classifier is currenlty processing values, in which case we don't want to tell it to do anything new or else it might get stuck in a infinite loop. */
    public boolean isProcessing();
}
