<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:java="http://xml.apache.org/xslt/java"
  xmlns:util="xalan://org.greenstone.gsdl3.util.XSLTUtil"
  extension-element-prefixes="java util"
  exclude-result-prefixes="java util">

  
  <xsl:include href="style.xsl"/>
  <xsl:include href="service-params.xsl"/>
  <xsl:include href="querytools.xsl"/>
  
  <xsl:output method="html"/>
  
  <xsl:template name="pageHead">
    <head>
      <title><xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'nzdl')"/></title>
    </head>
    <xsl:call-template name="headScript"/>
  </xsl:template>

  <xsl:template match="pageResponse">
    <xsl:variable name="collName"><xsl:value-of select="/page/pageRequest/paramList/param[@name='c']/@value"/></xsl:variable>

    <center>
      <xsl:call-template name="collectionPageBanner">
	<xsl:with-param name="collName" select="$collName"/>
	<xsl:with-param name="pageType">search</xsl:with-param>
      </xsl:call-template>
      <xsl:apply-templates select="service">
	<xsl:with-param name="collName" select="$collName"/>
      </xsl:apply-templates>
    </center>

    <xsl:if test="documentNodeList">
      <xsl:call-template name="query-response">
	<xsl:with-param name="collName" select="$collName"/>
      </xsl:call-template>
    </xsl:if>
    <center>
      <xsl:call-template name="greenBar"/>
    </center>
  </xsl:template>
  
  <xsl:template name="query-response">
    <xsl:param name="collName"/>
    <xsl:call-template name="greenBarResults"/>

    <!-- If query term information is available, display it -->
    <br/>
    <xsl:if test="count(termList/term) > 0">
      <small>
	<xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'query.wordcount')"/> 
	<xsl:for-each select="termList/term">
	  <xsl:if test="position() > 1">, </xsl:if>
	  <xsl:value-of select="@name"/>: <xsl:value-of select="@freq"/> 
	</xsl:for-each>
      </small>
      <br/>
    </xsl:if>
    
    <!-- If the number of matching documents is known, display it -->
    <xsl:variable name="numDocsMatched" select="metadataList/metadata[@name='numDocsMatched']/@value"/>
    <xsl:if test="$numDocsMatched">
      <xsl:choose>
	<xsl:when test="$numDocsMatched='0'">
	  <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'query.nodocsmatch')"/>
	</xsl:when>
	<xsl:when test="$numDocsMatched='1'">
	  <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'query.onedocsmatch')"/>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:value-of select="util:getInterfaceText($interface_name, /page/@lang, 'query.manydocsmatch', $numDocsMatched)"/>
	</xsl:otherwise>
      </xsl:choose>
    </xsl:if>
    
    <!-- Display the matching documents -->
    <xsl:variable name="hp"><xsl:choose><xsl:when test="/page/pageRequest/paramList/param[@name='hp']"><xsl:value-of select="/page/pageRequest/paramList/param[@name='hp']/@value"/></xsl:when><xsl:otherwise>20</xsl:otherwise></xsl:choose></xsl:variable>
    <xsl:variable name="hn" select="/page/pageRequest/paramList/param[@name='hn']/@value"/>      
    <table>
      <tr valign="top">
	<xsl:for-each select="documentNodeList/documentNode">
	  <xsl:if test="$hp=-1 or (position()&gt; (($hn - 1)*$hp) and position() &lt;= ($hn * $hp))">
	    <tr>
	      <xsl:apply-templates select=".">
		<xsl:with-param name="collName" select="$collName"/>
	      </xsl:apply-templates>
	    </tr>
	  </xsl:if>
	</xsl:for-each>
      </tr>
    </table>
    <!-- next and prev links at bottom of page -->
    <xsl:call-template name="result-navigation"><xsl:with-param name="library" select="$library_name"/></xsl:call-template>
  </xsl:template>


  <xsl:template match="service">
    <xsl:param name="collName"/>
    <xsl:variable name="subaction" select="../pageRequest/@subaction"/>
    <form name="QueryForm" method="get" action="/gsdl3/{$library_name}">
      <input type="hidden" name="a" value="q"/>
      <input type="hidden" name="sa" value="{$subaction}"/>
      <input type="hidden" name="rt" value="r"/>
      <input type="hidden" name="s" value="{@name}"/>
      <input type="hidden" name="c" value="{$collName}"/>
      <input type="hidden" name="hn" value="1"/>
      <xsl:apply-templates select="paramList"/>
    </form>
  </xsl:template>
  
  <!-- a special handling of the param list -->
  <xsl:template match="paramList">
    <table>
      <tr><td>
	  Search for <xsl:apply-templates select="param[@name='index']"><xsl:with-param name="default"><xsl:apply-templates select="param[@name='index']" mode="calculate-default"/></xsl:with-param></xsl:apply-templates><xsl:if test="param[@name='level']">at <xsl:apply-templates select="param[@name='level']"><xsl:with-param name="default"><xsl:apply-templates select="param[@name='level']" mode="calculate-default"/></xsl:with-param></xsl:apply-templates> level</xsl:if><xsl:call-template name='query_mode'/>
	</td></tr>
      <xsl:call-template name="query-and-submit"/>
    </table>
  </xsl:template>
  
  <!-- new template for match mode -->
  <xsl:template name='query_mode'>
    <xsl:variable name="qt" select="/page/pageRequest/paramList/param[@name='qt']/@value"/>
    <xsl:choose>
      <xsl:when test="$qt=1">
	<xsl:variable name="qfm" select="/page/pageRequest/paramList/param[@name='qfm']/@value"/>
	<xsl:choose>
	  <xsl:when test="$qfm=1">
	    and display results in <xsl:apply-templates select="param[@name='sortBy']"><xsl:with-param name="default"><xsl:apply-templates select="param[@name='sortBy']" mode="calculate-default"/></xsl:with-param></xsl:apply-templates> order
	  </xsl:when>
	  <xsl:otherwise>
	    which contain <xsl:apply-templates select="param[@name='matchMode']"><xsl:with-param name="default"><xsl:apply-templates select="param[@name='matchMode']" mode="calculate-default"/></xsl:with-param></xsl:apply-templates> of
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:when>
      <xsl:otherwise>
	<xsl:variable name="ct" select="/page/pageRequest/paramList/param[@name='ct']/@value"/>
	<xsl:variable name="qm" select="/page/pageRequest/paramList/param[@name='qm']/@value"/>
	<xsl:choose>
	  <xsl:when test="$qm=1 and $ct=0">
	    using ranked/bool query
	  </xsl:when>
	  <xsl:when test="$qm=1 and $ct=1">
	    and display results in <xsl:apply-templates select="param[@name='sortBy']"><xsl:with-param name="default"><xsl:apply-templates select="param[@name='sortBy']" mode="calculate-default"/></xsl:with-param></xsl:apply-templates> order
	  </xsl:when>
	  <xsl:otherwise>
	    which contain <xsl:apply-templates select="param[@name='matchMode']"><xsl:with-param name="default"><xsl:apply-templates select="param[@name='matchMode']" mode="calculate-default"/></xsl:with-param></xsl:apply-templates> of the words
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="param[@name='matchMode']" mode='query1'>
   <xsl:param name='default'/>
    in query 1
    <xsl:variable name="qfm" select="/page/pageRequest/paramList/param[@name='qfm']/@value"/>
    <xsl:choose>
      <xsl:when test="$qfm=1">
	and display results in <xsl:apply-templates select="../param[@name='sortBy']"><xsl:with-param name="default" select="$default"/></xsl:apply-templates> order
      </xsl:when>
      <xsl:otherwise>
	which contain <xsl:apply-templates select='.'><xsl:with-param name="default" select="$default"/></xsl:apply-templates> of the words
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="query-and-submit">
    <xsl:variable name="qt" select="/page/pageRequest/paramList/param[@name='qt']/@value"/>
    <xsl:choose>
      <xsl:when test="$qt=1"> <!-- doing form query -->
	<xsl:variable name="qfm" select="/page/pageRequest/paramList/param[@name='qfm']/@value"/>
	<xsl:choose>
	  <xsl:when test="$qfm=1">
	    <tr><td><xsl:apply-templates select="param[@name='complexField']"/></td></tr>
	  </xsl:when>
	  <xsl:otherwise>
	    <tr><td><xsl:apply-templates select="param[@name='simpleField']"/></td></tr>
	  </xsl:otherwise>
	</xsl:choose>
	<tr><td align='right'>
	    <input type="submit"><xsl:attribute name="value"><xsl:value-of select='../display/submit'/></xsl:attribute></input>
	  </td></tr>
      </xsl:when>
      <xsl:otherwise> <!-- doing text query -->
    <xsl:variable name="qb" select="/page/pageRequest/paramList/param[@name='qb']/@value"/>
	<xsl:choose>
	  <xsl:when test="$qb=1"><!-- large query box -->
	    <tr><td>
		<textarea name="q" cols='63' rows='10'>
		  <xsl:apply-templates select="param[@name='query']" mode="calculate-default"/><xsl:text> </xsl:text><!-- put a space here just in case there is no value- mozilla craps out if have a <textarea /> element -->
		</textarea>
	      </td></tr>
	    <tr><td align='right'>
		<input type="submit"><xsl:attribute name="value"><xsl:value-of select='../display/submit'/></xsl:attribute></input>
	      </td></tr>
	  </xsl:when>
	  <xsl:otherwise>
	    <tr><td>
		<xsl:variable name="qs"><xsl:apply-templates select="param[@name='query']" mode="calculate-default"/></xsl:variable>
		<nobr><xsl:apply-templates select="param[@name='query']"><xsl:with-param name="default" select="java:org.greenstone.gsdl3.util.XSLTUtil.tidyWhitespace($qs)"/></xsl:apply-templates><input type="submit"><xsl:attribute name="value"><xsl:value-of select='../display/submit'/></xsl:attribute></input></nobr>
	      </td></tr>
	  </xsl:otherwise>
	</xsl:choose>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template match="param" mode="calculate-default">
    <xsl:variable name="pname" select="@name"/>
    <xsl:choose><xsl:when test="/page/pageRequest/paramList/param[@name=$pname]"><xsl:value-of select="/page/pageRequest/paramList/param[@name=$pname]/@value"/></xsl:when><xsl:otherwise><xsl:value-of select="@default"/></xsl:otherwise></xsl:choose>
  </xsl:template>
  
  <xsl:template match="documentNode">
    <xsl:param name="collName"/>
    
    <td>
      <a><xsl:attribute name="href"><xsl:value-of select='$library_name'/>?a=d&amp;c=<xsl:value-of select='$collName'/>&amp;d=<xsl:value-of select='@nodeID'/><xsl:if test="@nodeType='leaf'">&amp;sib=1</xsl:if></xsl:attribute>
	<xsl:apply-templates select="." mode="displayNodeIcon"/>
      </a>
    </td>
    <td><xsl:value-of select="metadataList/metadata[@name='Title']"/></td>
  </xsl:template>
  
  
  <xsl:template name="greenBarResults">
    <p/>
    <center>
      <img src="interfaces/default/images/qryresb.gif" width="537" height="17"/>
    </center>
  </xsl:template>
  
</xsl:stylesheet>  
